@@ -1,9 +1,7 @@
 #' HTTP client
 #'
 #' @export
-#' @param url (character) A url. One of \code{url} or \code{handle} required.
-#' @param opts (list) curl options
-#' @param handle A handle
+#' @template args
 #' @details
 #' \strong{Methods}
 #'   \describe{
@@ -26,6 +24,7 @@
 #'       Make a HEAD request
 #'     }
 #'   }
+#'
 #' @format NULL
 #' @usage NULL
 #' @details Possible parameters (not all are allowed in each HTTP verb):
@@ -34,9 +33,11 @@
 #'  \item query - query terms, as a list
 #'  \item body - body as an R list
 #'  \item encode - one of form, multipart, json, or raw
-#'  \item disk - a path to write to. if NULL (default), memory used
+#'  \item disk - a path to write to. if NULL (default), memory used.
+#'  See \code{\link[curl]{curl_fetch_disk}} for help.
 #'  \item stream - an R function to determine how to stream data. if
-#'  NULL (default), memory used
+#'  NULL (default), memory used. See \code{\link[curl]{curl_fetch_stream}}
+#'  for help
 #'  \item ... curl options, only those in the acceptable set from
 #'  \code{\link[curl]{curl_options}} except the following: httpget, httppost,
 #'  post, postfields, postfieldsize, and customrequest
@@ -86,17 +87,23 @@ HttpClient <- R6::R6Class(
   public = list(
     url = NULL,
     opts = list(),
+    proxies = list(),
     headers = list(),
     handle = NULL,
 
     print = function(x, ...) {
       cat("<crul connection> ", sep = "\n")
-      cat(paste0("  url: ", if (is.null(self$url)) self$handle$url else self$url), sep = "\n")
-      cat("  options: ", sep = "\n")
+      cat(paste0("  url: ",
+                 if (is.null(self$url)) self$handle$url else self$url),
+          sep = "\n")
+      cat("  curl options: ", sep = "\n")
       for (i in seq_along(self$opts)) {
         cat(sprintf("    %s: %s", names(self$opts)[i],
                     self$opts[[i]]), sep = "\n")
       }
+      cat("  proxies: ", sep = "\n")
+      if (length(self$proxies)) cat(paste("    -", purl(self$proxies)),
+                                    sep = "\n")
       cat("  headers: ", sep = "\n")
       for (i in seq_along(self$headers)) {
         cat(sprintf("    %s: %s", names(self$headers)[i],
@@ -105,9 +112,15 @@ HttpClient <- R6::R6Class(
       invisible(self)
     },
 
-    initialize = function(url, opts, headers, handle) {
+    initialize = function(url, opts, proxies, headers, handle) {
       if (!missing(url)) self$url <- url
       if (!missing(opts)) self$opts <- opts
+      if (!missing(proxies)) {
+        if (!inherits(proxies, "proxy")) {
+          stop("proxies input must be of class proxy", call. = FALSE)
+        }
+        self$proxies <- proxies
+      }
       if (!missing(headers)) self$headers <- headers
       if (!missing(handle)) self$handle <- handle
       if (is.null(self$url) && is.null(self$handle)) {
@@ -128,7 +141,8 @@ HttpClient <- R6::R6Class(
         ),
         headers = self$headers
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -138,11 +152,6 @@ HttpClient <- R6::R6Class(
                     stream = NULL, encode = "multipart", ...) {
       curl_opts_check(...)
       url <- make_url(self$url, self$handle, path, query)
-      # opts <- list(post = TRUE)
-      # if (is.null(body)) {
-      #   opts$postfields <- raw(0)
-      #   opts$postfieldsize <- 0
-      # }
       opts <- prep_body(body, encode)
       rr <- list(
         url = url,
@@ -154,7 +163,8 @@ HttpClient <- R6::R6Class(
         headers = c(self$headers, opts$type),
         fields = opts$fields
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -179,7 +189,8 @@ HttpClient <- R6::R6Class(
         headers = self$headers,
         fields = body
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -204,7 +215,8 @@ HttpClient <- R6::R6Class(
         headers = self$headers,
         fields = body
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -229,7 +241,8 @@ HttpClient <- R6::R6Class(
         headers = self$headers,
         fields = body
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -248,7 +261,8 @@ HttpClient <- R6::R6Class(
         ),
         headers = self$headers
       )
-      rr$options <- utils::modifyList(rr$options, c(self$opts, ...))
+      rr$options <- utils::modifyList(rr$options,
+                                      c(self$opts, self$proxies, ...))
       rr$disk <- disk
       rr$stream <- stream
       private$make_request(rr)
@@ -278,7 +292,11 @@ HttpClient <- R6::R6Class(
         status_code = resp$status_code,
         request_headers = c(useragent = opts$options$useragent, opts$headers),
         response_headers = {
-          headers_parse(curl::parse_headers(rawToChar(resp$headers)))
+          if (grepl("^ftp://", resp$url)) {
+            list()
+          } else {
+            headers_parse(curl::parse_headers(rawToChar(resp$headers)))
+          }
         },
         modified = resp$modified,
         times = resp$times,
